; docformat = 'rst'
;+
; This is an example program to demonstrate how to create an MPG-4 movie with Coyote Graphics routines.
; This program required IDL 8.1 and higher to run.
;
; :Categories:
;    Graphics
;    
; :Examples:
;    Save the program as "mpg_movie.pro" and run it like this::
;       IDL> .RUN mpg_movie
;       
;    The movie can be played with any MPG-4 player. For example, Windows Media Player.
;       
; :Author:
;    FANNING SOFTWARE CONSULTING::
;       David W. Fanning 
;       1645 Sheely Drive
;       Fort Collins, CO 80526 USA
;       Phone: 970-221-0438
;       E-mail: david@idlcoyote.com
;       Coyote's Guide to IDL Programming: http://www.idlcoyote.com
;
; :History:
;     Change History::
;        Written, 26 February 2013 by David W. Fanning.
;        Updated for change in nCDF file format from time since 1-1-1 to time since 1800-1-1. 21 Oct 2015. DWF.
;
; :Copyright:
;     Copyright (c) 2013, Fanning Software Consulting, Inc.
;-
PRO MPG_Movie
 
   ; Set up variables for the plot. Normally, these values would be 
   ; passed into the program as positional and keyword parameters.
   ; Data file is the NCEP Reanalysis Monthly Mean Precipitation
   ; data set, available here: ftp://ftp.cdc.noaa.gov/Datasets/cmap/std/precip.mon.mean.nc.
   datafile = 'precip.mon.mean.nc'
   mpgFilename = 'monthly_precip_2010.mp4'
   
   ; To browse data file.
   ; nCDF_Browser, datafile
   
   ; Open the file and read the data. Variable names are case sensitive.
   file = Obj_New('ncdf_file', datafile)
   lats = file -> GetVarData('lat') 
   lons = file -> GetVarData('lon')
   time = file -> GetVarData('time') ; Hours since 1800-1-1 00:00:0.0
   rain = file -> GetVarData('precip', MissingIndices=missing)
   Obj_Destroy, file
   
   ; Make sure all missing values are set to 0. Find the contour levels.
   rain[missing] = 0.0
   divisions = 12
   levels = Indgen(12)
   
   ; Convert the time vector to a date string.
   timeOffset = Julday(1, 1, 1800, 0, 0, 0)
   julianDayTime = timeOffset + time / 24.0    
   timeString = cgJulian2Date(julianDayTime, Format=20) ; yyyymmdd
   
   ; We will animate precipitation for the year 2011.
   year = Fix(StrMid(timeString, 0, 4))
   month = Fix(StrMid(timeString, 4, 2))
   indices  = Where( year EQ 2010, count)
   IF count GT 0 THEN BEGIN
        precipMonths = rain[*,*,indices]
        year = year[indices]
   ENDIF
   
   ; Set up the video player for output.
   video = IDLffVideoWrite(mpgFilename, Format='mp4')
   framerate = 5
   framedims = [600, 450]
   stream = video.AddVideoStream(framedims[0], framedims[1], framerate)
   
   ; Load contour colors.
   ncolors = N_Elements(levels)
   cgLoadCT, 4, /Brewer, /Reverse, NColors=ncolors, Bottom=1
   TVLCT, cgColor('white', /Triple), 0
   TVLCT, cgColor('charcoal', /Triple), ncolors
   
   ; We are going to create high-resolution PNG images, which we will add
   ; to the video stream. We create the high-resolution PNG image from PostScript
   ; intermediate files.
   position = [0.1, 0.1, 0.9, 0.8]
   cgProgressBar = Obj_New("CGPROGRESSBAR", /Cancel, Title='Creating MPG-4 Movie...')
   cgProgressBar -> Start
   FOR j=0,count-1 DO BEGIN
    
      ; Create the PostScript file.
      cgPS_Open, 'movie.ps', /Quiet
      cgDisplay, 600, 450
      cgMap_Set, /Cylindrical, 0, 180, LIMIT=[Min(lats), Min(lons), Max(lats), Max(lons)], $
        Position=position
      charsize = (!D.Name EQ 'PS') ? cgDefCharsize()*0.5 : cgDefCharsize()*0.75
      cgMap_Grid, /Box_Axes, Charsize=charsize
      cgMap_Grid, Color='charcoal', GLinestyle=1
      cgContour, precipMonths[*,*,j], lons, lats, /Overplot, /Cell_Fill, $
         Levels=levels, C_Colors=Bindgen(ncolors), NoErase=(j NE 0), $
         Position=position
      cgMap_Grid, Color='charcoal', GLinestyle=1
      cgMap_Continents, Color='dark gray'
      fudge = (!D.Name EQ 'PS') ? 0.03 : 0.025
      cgText, position[0], position[3]+fudge, /Normal, StrTrim(year[j],2)
      cgColorbar, NColors=11, Bottom=1, OOB_Low=0B, OOB_High=Fix(ncolors), $
         Title= cgMonths(j+1) + ' Precipitation (mm/day)' , TLocation='Top', $
         Divisions=ncolors, /Discrete, Range=[Min(levels), Max(levels)], $
          Position=[0.25, 0.915, 0.75, 0.930], TCharsize=cgDefCharsize()          
      cgPS_Close, /PNG, Width=600, /Delete_PS ; Convert to PNG file.
      
      ; Read the high-resolution PNG file.
      image = Read_PNG('movie.png')
      File_Delete, 'movie.png'
      
      ; Add the high-resolution image to the video stream.
      void = video -> Put(stream, image)
      
      ; Did the user cancel the operaton?
      IF cgProgressBar -> CheckCancel() THEN BEGIN
         ok = Dialog_Message('The user cancelled operation.')
         video -> Cleanup
         RETURN
      ENDIF ELSE cgProgressBar -> Update, (j+1)/12.*100.
      
  ENDFOR
  
  ; Clean up by closing the video file and writing its location.
  cgProgressBar -> Destroy
  video -> Cleanup
  CD, Current=currentDir
  Print, 'File "' + mpgFilename + '" written to ' + currentDir + '.'

END ;*****************************************************************

; To create the MPG-4 movie file, run the program like this.

;  MPG_Movie
;    
;END